{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}save-error-modal{% endblock %}
{% block modal_title %}Error{% endblock %}

{% block modal_body %}
<div id="saveErrorMessage"></div>
{% endblock %}

{% block modal_footer %}
<span class="pull-left">&nbsp;</span>
<a class="btn btn-danger" data-dismiss="modal" href="#">OK</a>
{% endblock %}