{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}
{% load i18n %}

{% block modal_id %}select-template-modal{% endblock %}
{% block modal_title %}Data Entry{% endblock %}

{% block modal_body %}
<div class="alert alert-danger" id="banner_errors" style="display: none;">
    <a href="#" class="close" data-hide="alert" aria-label="close">&times;</a>
    <h4><i class="fas fa-exclamation-circle"></i> Error</h4>
    <div id="form_start_errors">
</div>
</div>
<div class="pop_curate_instruction">
    Please choose one of the following options to start curating a {% trans "record_label" %}:
</div>
<div id="form_start_content">
</div>
{% endblock %}

{% block modal_footer %}
<button class="btn btn-secondary pull-left" data-dismiss="modal"><i class="fas fa-times"></i> Cancel</button>
<div class="text-right">
    <button id="btn-save-data" class="btn btn-primary" hidden=""><i class="fas fa-save"></i> Save without editing</button>
    <button id="btn-open-data" class="btn btn-primary"><i class="fas fa-code"></i> Open in Text editor</button>
    <button id="btn-display-data" class="btn btn-primary"><i class="fas fa-play"></i> Start editing</button>
</div>
{% endblock %}