{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}
{% load i18n %}

{% block modal_id %}use-warning-modal{% endblock %}
{% block modal_title %}Validation Warning{% endblock %}

{% block modal_body %}
{% include 'core_curate_app/user/data-entry/modals/validation-warning.html' %}
<p>
Some <span style="color:red;">required</span> or <span style="color:blue;">recommended</span> fields are present but empty:
</p>
<div id="useErrorMessage">
</div>
<p>Edit the {% trans "form_label" %} or proceed to review page.</p>
{% endblock %}

{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fas fa-times"></i> Cancel</a>
<a href="{% url 'core_curate_view_data' data.data_structure.id %}" class="btn btn-primary proceed-review">
    <i class="fas fa-check"></i> Proceed to Review
</a>
{% endblock %}