<div class="container-fluid">
    <div class="row">
        <div class="col-10">
            <h2>{{ data.data_structure.name }}</h2>
        </div>
        {% include 'core_curate_app/common/tools_curate_data_structure.html' %}
    </div>
</div>
{% include 'core_curate_app/common/detail_curate_data_structure.html' %}