{% extends 'core_main_app/user/template/list_body.html' %}

{% block actions %}
<button class="btn btn-primary set-template" role="button"><i class="fas fa-play"></i> Select Template</button>
{% endblock %}
