{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}
{% load i18n %}
{% block modal_id %}download-modal{% endblock %}
{% block modal_title %}Download options{% endblock %}


{% block modal_body %}

<div>
    <div class="row">
        <div class="col-4">Download template:</div>
        <div class="col-7">
            <button class="btn btn-primary download-template-btn col-6">
                <i class="fas fa-file-code"></i> Download Template
            </button>
        </div>
    </div>
    <div class="row mt-1">
        <div class="col-4 pt-0">Download {% trans "form_label"|title %}:</div>
        <div class="col-7 pt-0">
            <button  class="btn btn-primary download-document-btn col-6">
                <i class="fas fa-file-alt"></i> Download {% trans "form_label"|title %}
            </button>
        </div>

    </div>

    <div class="mt-3">
        Click here to format content <input type="checkbox" id="format">
    </div>
</div>
{% endblock %}

{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fas fa-times"></i> Cancel</a>
{% endblock %}