{% extends 'core_main_app/_render/admin/theme/tools/box.html' %}

{% block box_title %}
    {{ data.data_structure.name }}
{% endblock %}

{% block box_tools %}
    {% include 'core_curate_app/common/tools_curate_data_structure.html' %}
{% endblock %}

{% block box_body %}
    {% include 'core_curate_app/common/detail_curate_data_structure.html' %}
{% endblock %}
