{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}xml-error-modal{% endblock %}
{% block modal_title %}Validation Error{% endblock %}

{% block modal_body %}
<div id="xmlErrorMessage"></div>
{% endblock %}

{% block modal_footer %}
<button class="btn btn-secondary" data-dismiss="modal"><i class="fas fa-times"></i> Close</button>
{% endblock %}